# Generated by Django 4.2.18 on 2025-03-23 19:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0022_alter_memberdevice_modified'),
    ]

    operations = [
        migrations.AddField(
            model_name='authsession',
            name='method',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=127, null=True),
        ),
    ]



from django.http import HttpResponse
from django.shortcuts import render
from django.db.models.query import QuerySet
from django.core.cache import cache
import hashlib

from rest import settings
from version import VERSION
import time
from auditlog.models import PersistentLog
from . import model as ms
from . import collection as cs
from . import json as js
from . import csv
from . import excel
# from . import profiler
STATUS_ON_PERM_DENIED = settings.get("STATUS_ON_PERM_DENIED", 403)
REST_LIST_CACHE_COUNT = settings.get("REST_LIST_CACHE_COUNT", False)


def get_query_hash(queryset):
    """Generate a unique hash based on the queryset's SQL query and params."""
    sql_query = str(queryset.query)
    return hashlib.sha256(sql_query.encode()).hexdigest()


def get_cached_count(queryset, hash=None, timeout=1800):
    """
    Retrieves the cached count for a queryset or calculates & caches it.
    """
    if hash is None:
        hash = get_query_hash(queryset)
    cache_key = f"rest_hash_{hash}"
    count = cache.get(cache_key)
    if count is None:
        count = queryset.count()
        cache.set(cache_key, count, timeout=timeout)
    return count


def restStatus(request, status, data={}, **kwargs):
    if isinstance(data, str):
        if status:
            data = dict(message=data)
        else:
            data = dict(error=data)
    data = data.copy()
    data.update(kwargs)
    data["status"] = status
    return restResult(request, data)


def restSuccessNoContent(request):
    return HttpResponse(status=204)


# @profiler.timeit
def restGet(request, obj, fields=None, extra=[], exclude=[], recurse_into=[], **kwargs):
    data = ms.to_dict(obj, fields, extra, exclude, recurse_into)
    if not kwargs.get("return_httpresponse", True):
        return data
    data = dict(data=data, status=kwargs.get("status", True))
    data["datetime"] = int(time.time())
    data["elapsed"] = get_request_elapsed(request)
    return restResult(request, data)


# @profiler.timeit
def restList(request, qset, size=25, start=0, sort=None, fields=None, extra=[], exclude=[], recurse_into=[], **kwargs):
    count = kwargs.get('count', 0)
    hash = None
    if count == 0:
        if isinstance(qset, QuerySet):
            if REST_LIST_CACHE_COUNT and not request.DATA.get("no_cache", False):
                hash = get_query_hash(qset)
                count = get_cached_count(qset, hash)
            else:
                count = qset.count()
        elif isinstance(qset, list):
            count = len(qset)
    start = request.DATA.get("start", start, field_type=int)
    size = request.DATA.get("size", size, field_type=int)
    sort = request.DATA.get("sort", sort)
    data = cs.to_list(qset, sort, size, start, fields, extra, exclude, recurse_into)
    data["status"] = kwargs.get("status", True)
    data["datetime"] = int(time.time())
    data["elapsed"] = get_request_elapsed(request)
    data["count"] = count
    if hash is not None:
        data["hash"] = hash
    if "response_params" in kwargs:
        for k, v in kwargs.get("response_params").items():
            data[k] = v
    return restResult(request, data)


def restResult(request, data, status=200):
    if "status" not in data:
        data["status"] = True
        data["datetime"] = int(time.time())
        data["elapsed"] = get_request_elapsed(request)
    accept_list = parse_accept_list(request)
    restResponseLog(request, data)
    if 'application/json' in accept_list:
        return HttpResponse(js.serialize(data), content_type="application/json", status=status)
    elif 'text/html' in accept_list:
        return render_webview(request, data, status)
    return HttpResponse(js.prettyJSON(data), content_type="text/plain", status=status)


restReturn = restResult  # legacy support


def restResponseLog(request, resp):
    # render response
    if hasattr(request, "rest_debug") and request.rest_debug:
        if settings.DEBUG_REST_NO_LISTS and "count" in resp:
            if "count" in resp:
                if settings.DEBUG_REST_NO_LIST == "example":
                    sanatized = dict(count=resp.get("count"), size=resp.get("size"), data=resp.get("data", [])[:1])
                else:
                    sanatized = dict(count=resp.get("count"), size=resp.get("size"))
                PersistentLog.log(sanatized, 0, request, "rest", action="response")
            else:
                PersistentLog.log(resp, 0, request, "rest", action="response")
        else:
            PersistentLog.log(resp, 0, request, "rest", action="response")


def restPermissionDenied(request, error="permission denied", error_code=403, status=STATUS_ON_PERM_DENIED,
                         component=None, component_id=None):
    # from rest import helpers as rh
    # rh.log_error(error)
    return restResult(request, dict(status=False, error=error, error_code=error_code), status=status)


def restNotFound(request):
    return restStatus(request, False, error="not found", error_code=404)


def restOK(request):
    return restStatus(request, True)


def restError(request, error, error_code=None):
    return restStatus(request, False, error=error, error_code=error_code)


def restJSON(request, qset, fields, name, size=10000, localize=None):
    if fields:
        data = cs.to_list(qset, fields=fields, size=size)
    else:
        data = cs.serialize(qset, request.DATA.get("graph"), size=size)
    response = HttpResponse(js.prettyJSON(data), content_type="application/json")
    response['Content-Disposition'] = 'attachment; filename="{0}"'.format(name)
    return response


def restFlat(request, qset, fields, name, size=10000, header_cols=None, localize=None):
    return HttpResponse(js.prettyJSON(list(qset.values_list(*fields))), content_type="application/json", status=200)


def restCSV(request, qset, fields, name, size=10000, header_cols=None, localize=None):
    if size > 1200 and qset.count() > 1200:
        # large data set lets stream
        qset = qset[:size]
        response = csv.generateCSVStream(qset, fields, name, localize=localize)
        response['Cache-Control'] = 'no-cache'
        response['Content-Disposition'] = f'attachment; filename="{name}"'
    else:
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = f'attachment; filename="{name}"'
        qset = qset[:size]
        csv.generateCSV(qset, fields, name, output=response, header_cols=header_cols, localize=localize)
    return response


def restExcel(request, qset, fields, name, size=10000, localize=None, **kwargs):
    return excel.qsetToExcel(request, qset[:size], fields, name)


def restHTML(request, html_content=None, template=None, context=None, status=200):
    if not html_content and not template:
        return HttpResponse("<html><body><h1>Hello, World!</h1><p>Welcome to my site.</p></body></html>")
    if template:
        return render(request, template, context, status=status)
    return HttpResponse(html_content, status=status)


def parse_accept_list(request):
    if request and hasattr(request, "DATA") and request.DATA.get('_type', None) is not None:
        accept_list = [request.DATA.get('_type')]
    elif request and 'HTTP_ACCEPT' in request.META:
        accept_list = request.META["HTTP_ACCEPT"].split(',')
    elif request and 'HTTP_ACCEPT_ENCODING' in request.META:
        accept_list = request.META["HTTP_ACCEPT_ENCODING"].split(',')
    else:
        accept_list = []
    return accept_list


def get_request_elapsed(request):
    if request is not None:
        return int((time.perf_counter() - request._started)*1000)
    return 0


def render_webview(request, data, status=200):
    output = js.prettyJSON(data)
    req_dict = request.DATA.asDict()
    req_out = js.prettyJSON(req_dict)
    path = request.path
    parts = path.split('/')
    last_pk = parts.pop()
    if last_pk and last_pk.isdigit():
        path = "/".join(parts)
    else:
        last_pk = ""

    is_debug = settings.get("DEBUG", False) or settings.get("REST_DEBUGGER", False)
    rest_help = {}
    if is_debug and hasattr(request, "rest_class"):
        rest_help = request.rest_class.get_rest_help()
        rest_help = js.prettyJSON(rest_help)
    context = {
        "help": rest_help,
        "input": req_dict,
        "pk": last_pk,
        "output_dict": data,
        "output": output,
        "req_out": req_out,
        "path": path,
        "method": request.method,
        "debug": is_debug,
        "request": request,
        "version": VERSION,
        "disclaimer": settings.REST_DISCLAIMER
    }
    return render(request, "rest_html.html", context, status=status)




from django.db import models
from rest import models as rm
from rest import fields as rf
from rest import settings
from objict import objict
from datetime import datetime

from rest import helpers as rh

CM_BACKENDS = objict()
DEVICE_USE_BUID = settings.get("DEVICE_USE_BUID", False)


class MemberDevice(models.Model, rm.RestModel, rm.MetaDataModel):
    """
    MemberDevice Model tracks personal devices associated with a user.
    This can include mobile and desktop devices.
    """
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True, db_index=True)

    member = models.ForeignKey("account.Member", related_name="devices", on_delete=models.CASCADE)

    name = models.CharField(max_length=128, blank=True, null=True, default=None)
    uuid = models.CharField(db_index=True, max_length=128, blank=True, null=True, default=None)
    buid = models.CharField(db_index=True, max_length=128, blank=True, null=True, default=None)

    cm_provider = models.CharField(db_index=True, max_length=64, default="fcm")
    cm_token = models.CharField(max_length=250, default=None, null=True)

    kind = models.CharField(db_index=True, max_length=64, default="unknown")
    ip = models.CharField(max_length=64, default=None, null=True)
    state = models.IntegerField(db_index=True, default=1)

    class RestMeta:
        GRAPHS = {
            "default": {
                "extra": ["metadata"],
            }
        }

    def sendData(self, message, **kwargs):
        messenger = getCloudMessenger(self.cm_provider)
        if messenger:
            resp = messenger.sendToDevice(self, message)
            return resp
        return objict(status_code=404, reason=self.cm_provider)

    def sendNotification(self, title, body):
        messenger = getCloudMessenger(self.cm_provider)
        if messenger:
            resp = messenger.sendNotification(self.cm_token, title, body)
            return resp
        return objict(status_code=404, reason=self.cm_provider)

    def notify(self, title, body):
        return self.sendNotification(title, body)

    def touch(self, ip=None):
        if ip is not None:
            self.ip = ip
        self.modified = datetime.now()
        self.save()

    @classmethod
    def sendMessageTo(cls, message, devices, **kwargs):
        pass

    @classmethod
    def register(cls, request, member, device_id):
        cm_token = request.DATA.get("cm_token")
        default_provider = "ws"
        if cm_token is not None:
            default_provider = "fcm"
        cm_provider = request.DATA.get("cm_provider", default=default_provider)
        buid = request.DATA.get("__buid__", None)
        md = MemberDevice.objects.filter(uuid=device_id, member=member).last()
        if md is not None:
            if md.cm_token != cm_token:
                md.cm_token = cm_token
            if buid and md.buid != buid:
                md.buid = buid
            if cm_provider is not None:
                md.cm_provider = cm_provider
            if md.state != 1:
                md.state = 1
            md.touch(request.ip)
            metadata = request.DATA.get("device_metadata", None)
            rh.debug("md.metadata", metadata)
            if metadata is not None:
                md.setProperty("device", metadata)
            return md
        md = MemberDevice(
            uuid=device_id, buid=buid,
            member=member, cm_token=cm_token,
            cm_provider=cm_provider)
        md.ip = request.ip
        md.name = F"{member.first_name} {request.auth_session.device}"
        md.kind = request.auth_session.os.lower()
        if md.kind.startswith("mac"):
            md.kind = "mac"
        md.save()
        md.setProperty("user_agent", request.META.get('HTTP_USER_AGENT', ''))
        metadata = request.DATA.get("device_metadata", None)
        if metadata is not None:
            md.setProperty("device", metadata)
        return md

    @classmethod
    def unregister(cls, member, device_id):
        md = MemberDevice.objects.filter(uuid=device_id, member=member, state=1).last()
        if md is None:
            return False
        md.cm_token = None
        md.state = -1
        md.save()
        return True


class MemberDeviceMetaData(rm.MetaDataBase):
    parent = models.ForeignKey(MemberDevice, related_name="properties", on_delete=models.CASCADE)


class CloudCredentials(models.Model, rm.RestModel, rm.MetaDataModel):
    """
    CloudCredentials is a global setting for a group to store this groups cloud credentials.
    """
    class RestMeta:
        VIEW_PERMS = ["view_cm", "manage_cm"]
        EDIT_PERMS = ["manage_cm"]
        GRAPHS = {
            "default": {
                "extra": ["metadata"],
                "graphs": {
                    "group": "basic"
                }
            }
        }

    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    group = models.ForeignKey(
        "account.Group", null=True, default=None,
        related_name="cloud_credentials", on_delete=models.CASCADE)

    name = models.CharField(max_length=128, blank=True, null=True, default=None)
    uuid = models.CharField(db_index=True, max_length=64, blank=True, null=True, default=None)
    state = models.IntegerField(db_index=True, default=1)

    # specify the cloud provider class "account.fcm.v1.FirebaseNotifier"
    # provider_class = models.CharField(max_length=255, null=True, default=None)

    credentials = rf.JSONField()

    _notifier = None

    @property
    def notifier(self):
        if self._notifier is None:
            from account import fcm
            self._notifier = fcm.v1.FirebaseNotifier(self.credentials) 
        return self._notifier

    def sendToDevice(self, device, message):
        return self.notifier.send(device.cm_token, None, None, message)

    def sendNotification(self, token, title, body):
        return self.notifier.send(token, title, body)


class CloudCredentialsMetaData(rm.MetaDataBase):
    parent = models.ForeignKey(CloudCredentials, related_name="properties", on_delete=models.CASCADE)


def getCloudMessenger(name):
    creds = CloudCredentials.objects.filter(uuid=name).last()
    if creds is not None:
        return creds
    return None



# Generated by Django 4.2.11 on 2025-01-03 19:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0015_rule_title_template_alter_incident_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='rule',
            name='notify_template',
            field=models.TextField(default=None, null=True),
        ),
    ]



import os
import signal
import time
import argparse
import logging
import daemon
from daemon import pidfile
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler


class Service(FileSystemEventHandler):
    def __init__(self, name, pid_file, watch_file=None, require_user=None, logger=None):
        self.name = name
        self.pid_file = pid_file
        self.watch_file = os.path.realpath(watch_file) if watch_file else None
        self.require_user = require_user
        self.observer = None

    def log_info(self, *args):
        if self.logger:
            self.log_info(*args)

    def start(self):
        if self.is_running():
            self.log_info("Service is already running.")
            return

        self.log_info(f"Starting {self.name}...")

        with daemon.DaemonContext(
            pidfile=pidfile.TimeoutPIDLockFile(self.pid_file),
            stdout=open('/tmp/service.log', 'a+'),
            stderr=open('/tmp/service_error.log', 'a+'),
            signal_map={
                signal.SIGTERM: self.stop,
                signal.SIGHUP: self.restart
            }
        ):
            self.run()

    def stop(self, *args):
        pid = self.get_pid()
        if not pid:
            self.log_info("Service is not running.")
            return

        self.log_info(f"Stopping {self.name} (PID: {pid})...")
        try:
            os.kill(pid, signal.SIGTERM)
            time.sleep(1)
            if self.is_running():
                os.kill(pid, signal.SIGKILL)
            if os.path.exists(self.pid_file):
                os.remove(self.pid_file)
            self.log_info("Service stopped.")
        except OSError:
            self.logger.error(f"Failed to stop process {pid}")

    def restart(self, *args):
        self.log_info("Restarting service...")
        self.stop()
        time.sleep(1)
        self.start()

    def status(self):
        if self.is_running():
            print(f"{self.name} is running (PID: {self.get_pid()})")
        else:
            print(f"{self.name} is not running")

    def is_running(self):
        """Check if the process is running using os.kill(pid, 0)"""
        pid = self.get_pid()
        if pid:
            try:
                os.kill(pid, 0)  # Does not kill, just checks if process exists
                return True
            except OSError:
                return False
        return False

    def get_pid(self):
        if os.path.exists(self.pid_file):
            try:
                with open(self.pid_file, "r") as f:
                    pid = int(f.read().strip())
                return pid
            except ValueError:
                return None
        return None

    def run(self):
        """Main daemon loop"""
        self.log_info(f"{self.name} is running in background.")
        if self.watch_file:
            self.start_watcher()
        self.on_run()

    def on_run(self):
        while True:
            time.sleep(5)

    def start_watcher(self):
        self.log_info(f"Watching file: {self.watch_file}")
        self.observer = Observer()
        self.observer.schedule(self, path=os.path.dirname(self.watch_file), recursive=False)
        self.observer.start()

    def on_modified(self, event):
        if self.watch_file and event.src_path == self.watch_file:
            self.log_info("Config file changed, restarting service...")
            self.restart()


def main():
    parser = argparse.ArgumentParser(description="Manage the service daemon")
    parser.add_argument("command", choices=["start", "stop", "restart", "status", "run"])
    args = parser.parse_args()

    service = Service(name="MyDaemon", pid_file="/tmp/mydaemon.pid")

    if args.command == "start":
        service.start()
    elif args.command == "stop":
        service.stop()
    elif args.command == "restart":
        service.restart()
    elif args.command == "status":
        service.status()
    elif args.command == "run":
        service.run()

if __name__ == "__main__":
    main()
